#!/bin/bash
# Copyright 2015 Michal Žejdl
# one-time checker for SNMP availability on IPv4 and IPv6 addresses
#	based on rresolver.sh
# called by: start.sh (as a golem)
# input: arp, nd (sql tables)
# output: snmp4, snmp6 (sql tables)
# dependence: dirname, mysql, snmpget

confdir=$(dirname $0)/..
config=$confdir/defaults.sh
test -f $config || exit
. $config

# smaller limit leads to heavier mysql load
limit=100

# a new file descriptor to prevent pipe from closing
exec 3> $mysqlwpipe

do_query () {
	test -z "$1" && return 1
	result=($(mysql -e "$1" $select_opts)) || return 1
}

# is in arp and not in snmp4
get_new4 () {
	do_query "select distinct arp.ip from arp left join snmp4 on arp.ip = snmp4.address where snmp4.address is null order by arp.time desc limit $limit" || return 1
}

# is in nd and not in snmp6
get_new6 () {
	# select only global (2000::/3); how to avoid RFC 4941 addresses?
	do_query "select distinct nd.ip6 from nd left join snmp6 on nd.ip6 = snmp6.address where snmp6.address is null and (nd.ip6 like '2%' or nd.ip6 like '3%') order by nd.time desc limit $limit" || return 1
}

# check addresses which had SNMP agent in the past - the most recent goes first
get_used4 () {
	do_query "select address from snmp4 where status != 0 and get is not null order by cast(time - get as signed) limit $limit" || return 1
}

get_used6 () {
	do_query "select address from snmp6 where status != 0 and get is not null order by cast(time - get as signed) limit $limit" || return 1
}

# the oldest nonsnmp in snmp4 with a new arp entry 
get_nonsnmp_live4 () {
	do_query "select distinct address from snmp4 join arp on snmp4.address = arp.ip where snmp4.status != 0 and snmp4.time < arp.time order by snmp4.time limit $limit" || return 1
}

# the oldest nonsnmp in snmp6 with a new nd entry 
get_nonsnmp_live6 () {
	do_query "select distinct address from snmp6 join nd on snmp6.address = nd.ip6 where snmp6.status != 0 and snmp6.time < nd.time order by snmp6.time limit $limit" || return 1
}

# the oldest nonsnmp in snmp4 without a new arp entry
get_nonsnmp_dead4 () {
	do_query "select distinct address from snmp4 join arp on snmp4.address = arp.ip where snmp4.status != 0 and snmp4.time >= arp.time order by snmp4.time limit $limit" || return 1
}

# the oldest nonsnmp in snmp6 without a new nd entry
get_nonsnmp_dead6 () {
	do_query "select distinct address from snmp6 join nd on snmp6.address = nd.ip6 where snmp6.status != 0 and snmp6.time >= nd.time order by snmp6.time limit $limit" || return 1
}

# input: IP address, version (4 or 6); output: insert/update snmp$version table
update () {
	if test $2 -eq 6; then
		snmpaddress=udp6:[$1]
	else
		snmpaddress=$1
	fi
	# without MIB (fast), one shot, sysObjectID.0
	snmpget -m '' -v 1 -r 0 -c $snmprcommunity $snmpaddress .1.3.6.1.2.1.1.2.0 &>/dev/null
	status=$?
	# echo should be fflushed automaticaly according to http://stackoverflow.com/questions/9926616/is-echo-atomic-when-writing-single-lines
	if test $status -eq 0; then
		echo "insert snmp$2 (address, time, status, agent, get) values ('$1', unix_timestamp(), $status, null, unix_timestamp()) on duplicate key update time = unix_timestamp(), status = $status, agent = null, get = unix_timestamp();" >&3
	else
		echo "insert snmp$2 (address, time, status, agent, get) values ('$1', unix_timestamp(), $status, null, null) on duplicate key update time = unix_timestamp(), status = $status;" >&3
	fi
}

while true; do
	for method in new used nonsnmp_live nonsnmp_dead; do
		for version in 4 6; do
			if get_$method$version; then
				for ((ri = 0; ri < ${#result[@]}; ri++)); do
  					# stop this golem if "šém" is removed
					test -e $rundir/snmp/$(basename $0 .sh).sem || exit
					update "${result[$ri]}" "$version"
					# sleep only if snmpget succeeds
					# otherwise the SNMP timeout do the job
					test $status -eq 0 && sleep 1
				done
			fi
		done
	done
done
